@@ -23,7 +23,7 @@ int main( int /*argc*/, char** /*argv*/ )
     if( t )
     {
       ++fail;
-      printf( "test '%s' failed\n", name.c_str() );
+      fprintf( stderr, "test '%s' failed\n", name.c_str() );
     }
     delete t;
     t = 0;
@@ -37,7 +37,7 @@ int main( int /*argc*/, char** /*argv*/ )
     if( t->xml() != "<request xmlns='"+ XMLNS_RECEIPTS + "'/>" )
     {
       ++fail;
-      printf( "test '%s' failed: %s\n", name.c_str(), t->xml().c_str() );
+      fprintf( stderr, "test '%s' failed: %s\n", name.c_str(), t->xml().c_str() );
     }
     delete t;
     t = 0;
@@ -51,7 +51,7 @@ int main( int /*argc*/, char** /*argv*/ )
     if( t->xml() != "<received xmlns='"+ XMLNS_RECEIPTS + "'/>" )
     {
       ++fail;
-      printf( "test '%s' failed: %s\n", name.c_str(), t->xml().c_str() );
+      fprintf( stderr, "test '%s' failed: %s\n", name.c_str(), t->xml().c_str() );
     }
     delete t;
     t = 0;
@@ -70,7 +70,7 @@ int main( int /*argc*/, char** /*argv*/ )
     if( se == 0 || se->rcpt() != Receipt::Request )
     {
       ++fail;
-      printf( "test '%s' failed\n", name.c_str() );
+      fprintf( stderr, "test '%s' failed\n", name.c_str() );
     }
     delete f;
   }
@@ -86,7 +86,7 @@ int main( int /*argc*/, char** /*argv*/ )
     if( se == 0 || se->rcpt() != Receipt::Received )
     {
       ++fail;
-      printf( "test '%s' failed\n", name.c_str() );
+      fprintf( stderr, "test '%s' failed\n", name.c_str() );
     }
     delete f;
   }
@@ -99,7 +99,7 @@ int main( int /*argc*/, char** /*argv*/ )
   }
   else
   {
-    printf( "%d test(s) failed\n", fail );
+    fprintf( stderr, "%d test(s) failed\n", fail );
     return 1;
   }
 
